/*
 * Performance-monitoring counters driver.
 * Compatibility definitions for 2.2.x/2.3.x kernels.
 *
 * Copyright (C) 1999-2000  Mikael Pettersson
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,25)
#define vma_offset(vma)		((vma)->vm_offset)
#else
#define vma_offset(vma)		((vma)->vm_pgoff << PAGE_SHIFT)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,23)
#define get_zeroed_page(mask)	get_free_page((mask))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,13) && defined(MODULE)
#define __cacheline_aligned __attribute__((__aligned__(SMP_CACHE_BYTES)))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,11)
#define X86_CR4_TSD	0x0004
#define X86_CR4_PCE	0x0100
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,9)
#define get_file(x)	((x)->f_count++)
#define file_count(x)	((x)->f_count)
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,3,10)
#define get_file(x)	atomic_inc(&(x)->f_count)
#define file_count(x)	atomic_read(&(x)->f_count)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,6)
static __inline__ struct dentry *my_d_alloc_root(struct inode *inode)
{
	return d_alloc_root(inode, NULL);
}
#undef d_alloc_root	/* in case CONFIG_MODVERSIONS renamed it */
#define d_alloc_root(inode)	my_d_alloc_root((inode))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,1)
#define DECLARE_MUTEX(name)	struct semaphore name = MUTEX
#endif
