/*
 * x86 Performance-Monitoring Counters driver
 *
 * Copyright (C) 1999-2000  Mikael Pettersson
 */
#ifndef _ASM_I386_PERFCTR_H
#define _ASM_I386_PERFCTR_H

struct perfctr_sum_ctrs {
	unsigned long long tsc;
	unsigned long long pmc[4];
};

struct perfctr_low_ctrs {
	unsigned int tsc;
	unsigned int pmc[4];
};

struct perfctr_control {
	unsigned int evntsel[4];
};

#ifdef __KERNEL__

#if defined(CONFIG_PERFCTR) || defined(CONFIG_PERFCTR_MODULE)

/* called from arch/i386/kernel/setup.c:dodgy_tsc() */
extern void perfctr_dodgy_tsc(void);

/* Initialise the hardware driver. */
extern int perfctr_cpu_init(void);

#ifdef CONFIG_PERFCTR_MODULE
/* Clean up hardware state before module unload. */
extern void perfctr_cpu_cleanup(void);
#endif

/* Reserve the hardware for a particular service. */
extern const char *perfctr_cpu_reserve(const char *service);
extern void perfctr_cpu_release(const char *service);

/* Save current TSC/PMC values in *ctrs. */
extern void (*perfctr_cpu_read_low_counters)(int nrctrs, struct perfctr_low_ctrs *ctrs);

/* Write control data to the CPU's perfctr evntsel MSRs.
   This is to be done lazily: if a given evntsel MSR already
   contains the correct value, do not write to it. */
extern void (*perfctr_cpu_write_control)(int nrctrs, const struct perfctr_control *control);

/* Return -EPERM if the control info is invalid, otherwise the
   highest numbered enabled counter (starting from 1). */
extern int (*perfctr_cpu_check_control)(const struct perfctr_control *control);

/* Detected CPU type. */
extern unsigned char perfctr_cpu_type;

/* CPU features optionally supported. */
extern unsigned char perfctr_cpu_features;

/* Process resume/suspend hooks used for virtual perfctrs.
 * These are now no-ops on x86 since we keep CR4.PCE
 * globally enabled while the driver is loaded.
 */
static __inline__ void perfctr_cpu_enable_rdpmc(void) { }
static __inline__ void perfctr_cpu_disable_rdpmc(void) { }

#else	/* !CONFIG_PERFCTR */

static __inline__ void perfctr_dodgy_tsc(void) { }

#endif	/* CONFIG_PERFCTR */

#endif	/* __KERNEL__ */

#endif	/* _ASM_I386_PERFCTR_H */
